from flask import Blueprint, current_app, request
from flask import render_template, abort
from flask_login import current_user
from srht.config import cfg
from srht.flask import paginate_query
from srht.search import search
from scmsrht.types import Access, Repository, User
from scmsrht.repos.repository import RepoVisibility
from sqlalchemy import and_, or_

public = Blueprint('public', __name__)

@public.route("/")
def index():
    if current_user:
        repos = (Repository.query
                .filter(Repository.owner_id == current_user.id)
                .filter(Repository.visibility != RepoVisibility.autocreated)
                .order_by(Repository.updated.desc())
                .limit(10)).all()
        return render_template("dashboard.html", repos=repos)
    return render_template("index.html")

@public.route("/~<username>")
@public.route("/~<username>/")
def user_index(username):
    user = User.query.filter(User.username == username).first()
    if not user:
        abort(404)
    terms = request.args.get("search")
    repos = (Repository.query
            .filter(Repository.owner_id == user.id))
    if current_user and current_user.id != user.id:
        repos = (repos
                .outerjoin(Access._get_current_object(),
                    Access.repo_id == Repository.id)
                .filter(or_(
                    Access.user_id == current_user.id,
                    and_(
                        Repository.visibility == RepoVisibility.public,
                        Access.id.is_(None))
                )))
    elif not current_user:
        repos = repos.filter(Repository.visibility == RepoVisibility.public)
    if terms:
        repos = search(repos, terms,
                [Repository.name, Repository.description], {})
    repos = repos.order_by(Repository.updated.desc())
    repos, pagination = paginate_query(repos)

    return render_template("user.html",
            user=user, repos=repos,
            search=terms, **pagination)
